import { ViteDevServer, Plugin } from 'vite';
import { PageDirOptions } from 'vite-plugin-files';

interface RouteFile {
    path: string;
    file: string;
    exact?: boolean;
    routes?: RouteFile[];
}
interface RouteDirOptions extends PageDirOptions {
    baseRoute: string;
}
/**
 * Plugin options.
 */
interface Options {
    /**
     * Relative path to the directory to search for page components.
     * @default 'src/pages'
     */
    routesDir: RouteDirOptions[];
    /**
     * file extension
     * @default ["tsx"]
     */
    extensions: string[];
    /**
     * List of path globs to exclude when resolving files.
     * @default ["components"]
     */
    exclude: RegExp[];
    /**
     * Enable or not HashRouter .
     * @default false
     */
    hash: boolean;
}
declare type UserOptions = Partial<Options>;
interface ResolvedOptions extends Options {
    devServer?: ViteDevServer;
}

declare function VitePluginReactRouter(userOptions: UserOptions): Plugin;

export default VitePluginReactRouter;
export { ResolvedOptions, RouteDirOptions, RouteFile, UserOptions };
